<!-- !This file uses the Genshi template language: http://genshi.edgewall.org/wiki/Documentation/xml-templates.html -->
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:py="http://genshi.edgewall.org/">
    <head>
        <title>$title</title>

        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />

        <link rel="stylesheet" type="text/css" media="screen" href="${root}/.static/css/default.css" />
        <link rel="alternate" type="application/thredds+xml" title="THREDDS Catalog" href="${root}/catalog.xml" />

        <!-- icons from http://www.famfamfam.com/lab/icons/silk/preview.php -->
    </head>

    <body>
        <div id="main">
            <h1>$title</h1>

            <hr />
            <p><a href="../"><img src="${root}/.static/images/folder_explore.png" />Parent directory</a></p>
            <table>
                <thead>
                    <th>Filename</th>
                    <th>Download</th>
                    <th>Last modified</th>
                </thead>

                <!-- diretories -->
                <tr py:for="dir in dirs">
                    <td><a href="${dir}/"><img src="${root}/.static/images/folder.png" />${dir}/</a></td><td colspan="2"></td>
                </tr>

                <!-- files -->
                <tr py:for="file in files">
                    <td py:choose="">
                        <a py:when="file.supported" href="${file.name}.html" title="Download data from ${file.name}">
                            <img src="${root}/.static/images/page_white_code.png" />${file.name}</a>
                        <py:otherwise><img src="${root}/.static/images/page_white.png" />${file.name}</py:otherwise>
                    </td>
                    <td><a href="${file.name}" title="Download file ${file.name}">${file.size}</a></td> 
                    <?python
                        import time
                    ?>
                    <td>${time.strftime("%m/%d/%Y %I:%M:%S %p", file.modified)}</td> 
                </tr>
            </table>

            <hr />
            <p class="footnote"><em><a href="http://pydap.org/">pydap/$version</a></em></p>
        </div>
    </body>
</html>

<!--! vim:ft=genshi
-->
