body {
    background-color: #eee;
    font-family: Verdana,Arial,sans-serif;
    font-size: 76%;
    margin: 0;
    padding: 0;
}

a {
    color: #222;
}

hr {
    border: 0;
    height: 1px;
    color: #ccc;
    background-color: #ccc;
}

img {
    border: 0;
    vertical-align: middle;
    margin-right: 4px;
}

div {
    margin: 0;
    padding: 0;
}

div#main {
    background-color: #fff;
    border: 1px solid #ccc;
    -moz-border-radius: 10px 10px;
    border-radius: 10px 10px;
    margin: 20px;
    padding: 10px 20px;
}

h1 {
    font-size: 1.4em;
}

h2 {
    font-size: 1.2em;
}

table { 
    font-size: 1.1em;
    width: 100%;
}

th {
    text-align: left;
}

th[colspan] {
    text-align: center;
}

fieldset {
    border: 1px solid #ccc;
    -moz-border-radius: 10px 10px;
    border-radius: 10px 10px;
    margin: 20px 0;
    padding: 0 20px 10px 20px;
}

legend {
    font-weight: bold;
    margin-left: 10px;
}

dl {
    background: #eee;
    padding: 10px 20px;
}

dt {
    font-weight: bold;
}

dd {
}

p.footnote {
    text-align: right;
}

pre {
    margin: 10px 20px;
}

code {
    font-size: 1.2em;
}

#map {
    border: 1px solid #ccc;
    width: 600px;
    height: 300px;
}
